;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PIC16F628A. This file contains the basic code              *
;   building blocks to build upon.                                    *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PIC data sheet for additional             *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    16F628ATEMP.asm                                   *
;    Date:          08/2011                                           *
;    File Version:  1.0                                               *
;                                                                     *
;    Author:        Kovalev Anton YUrievich                           *
;    Company:       A                                                 *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files Required: P16F628A.INC                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:         Reaction                                          *
;                                                                     *
;**********************************************************************

	list      p=16f628A           ; list directive to define processor
	#include <p16F628A.inc>       ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_ON & _XT_OSC 

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.

    #define	BUTTON  PORTA,0
    #define	LED     PORTB,2
    #define	LCD_RS  PORTB,0
    #define	LCD_E   PORTB,1
    #define	LCD_DB4 PORTB,4
    #define	LCD_DB5 PORTB,5
    #define	LCD_DB6 PORTB,6
    #define	LCD_DB7 PORTB,7

;***** VARIABLE DEFINITIONS
w_temp        EQU     0x7E        ; variable used for context saving 
status_temp   EQU     0x7F        ; variable used for context saving

CONTROL      equ 0x20
PAUSE_1      equ 0x21
PAUSE_2      equ 0x22
PAUSE_3      equ 0x23
PAUSE_LCD_1  equ 0x24
PAUSE_LCD_2  equ 0x25
LCD_BUFFER   equ 0x26
LCD_CYCLE    equ 0x27
COUNT        equ 0x28  
TIME_L       equ 0x29
TIME_H       equ 0x2A
TIME_1_L     equ 0x2B
TIME_1_H     equ 0x2C
TIME_2_L     equ 0x2D
TIME_2_H     equ 0x2E 
COMP_1_L     equ 0x2F
COMP_1_H     equ 0x30
COMP_2_L     equ 0x31
COMP_2_H     equ 0x32
VL           equ 0x33
VH           equ 0x34
D0           equ 0x35 
D1           equ 0x36
D2           equ 0x37 
D3           equ 0x38
D4           equ 0x39
DD           equ 0x3A  
RADR         equ 0x3B
RDATA        equ 0x3C

;**********************************************************************
	ORG     0x000             ; processor reset vector
	goto    main              ; go to beginning of program
	
	ORG     0x004             ; interrupt vector location
	movwf   w_temp            ; save off current W register contents
	movf	STATUS,w          ; move status register into W register
	movwf	status_temp       ; save off contents of STATUS register

; isr code can go here or be located as a call subroutine elsewhere

	movf    status_temp,w     ; retrieve copy of STATUS register
	movwf	STATUS            ; restore pre-isr STATUS register contents
	swapf   w_temp,f
	swapf   w_temp,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt

;**********************************************************************
; CONTROL
; 0 bit - Button ON
; 1 bit - Button ON -> OFF
; 2 bit - Pause 2 -> END
; 3 bit - Cycle Time -> END 
; 4 bit - Compare -> Yes
;**********************************************************************
main              call   Clear_SRAM
                  call   EEPROM_Load_R
                  call   First_Load_TIME_2
                  call   Ports_Initializing
                  call   TMR1_Initializing
                  call   PWM_Initializing
                  call   LCD_Initializing
                  call   Pause_LCD 
                  call   LCD_Initializing
                  call   Pause_LCD
                  bsf    LED
                  goto   Run_Record 
;**********************************************************************
;***************************** Clear SRAM *****************************
;**********************************************************************
Clear_SRAM        clrf CONTROL  
                  clrf PAUSE_1      
                  clrf PAUSE_2      
                  clrf PAUSE_3      
                  clrf PAUSE_LCD_1
                  clrf PAUSE_LCD_2   
                  clrf LCD_BUFFER   
                  clrf LCD_CYCLE    
                  clrf COUNT          
                  clrf TIME_L       
                  clrf TIME_H       
                  clrf TIME_1_L     
                  clrf TIME_1_H     
                  clrf TIME_2_L     
                  clrf TIME_2_H      
                  clrf COMP_1_L     
                  clrf COMP_1_H     
                  clrf COMP_2_L     
                  clrf COMP_2_H     
                  clrf VL           
                  clrf VH           
                  clrf D0            
                  clrf D1           
                  clrf D2            
                  clrf D3           
                  clrf D4           
                  clrf DD             
                  clrf RADR         
                  clrf RDATA                                
                  return  
;**********************************************************************
;************************* First Load TIME 2 **************************
;**********************************************************************
First_Load_TIME_2  movlw  0x0F
                   movwf  TIME_2_L
                   movlw  0x27
                   movwf  TIME_2_H
                   return
;**********************************************************************
;************************* Ports Initializing *************************
;**********************************************************************
Ports_Initializing clrf    PORTA
                   clrf    PORTB
                   clrf    CCP1CON
                   movlw   0x07
                   movwf   CMCON 
                   bsf     STATUS,RP0
                   bcf     STATUS,RP1
                   movlw   0x00
                   movwf   VRCON 
                   movlw   0x01
                   movwf   TRISA  
                   movlw   0x00
                   movwf   TRISB 
                   bcf     STATUS,RP0
                   return      
;**********************************************************************
;************************* TMR1 Initializing **************************
;**********************************************************************
TMR1_Initializing  clrf   T1CON
                   clrf   TMR1L
                   clrf   TMR1H
                   clrf   INTCON
                   bsf    STATUS,RP0
                   clrf   PIE1
                   bcf    STATUS,RP0
                   clrf   PIR1
                   movlw  0x20
                   movwf  T1CON
                   bcf    T1CON,TMR1ON 
                   return
;**********************************************************************
;************************** PWM Initializing **************************
;**********************************************************************
PWM_Initializing clrf    CCP1CON     
                 clrf    TMR2
                 movlw   0x7F 
                 movwf   CCPR1L    
                 clrf    INTCON    
                 bsf     STATUS,RP0
                 bcf     TRISB,3   
                 clrf    PIE1
                 movlw   0xFF
                 movwf   PR2
                 bcf     STATUS,RP0                                        
                 clrf    PIR1
                 movlw   0x2C
                 movwf   CCP1CON
                 movlw   0x00
                 movwf   T2CON
                 return 
;**********************************************************************
;***************************** Run Record ***************************** 
;**********************************************************************
Run_Record         call   LCD_Record
                   bcf    T2CON,TMR2ON
                   call   Pause_1
                   bsf    T2CON,TMR2ON
                   call   Pause_1
                   bcf    T2CON,TMR2ON
                   call   Pause_1
;**********************************************************************
;************************** Run Pause Start ***************************
;**********************************************************************
Run_Pause_Start    bcf    CONTROL,0
                   bcf    CONTROL,1
Run_Pause_Start_1  movlw  0x01
                   call   Button
                   btfsc  CONTROL,1
                   goto   Run_Pause_Start_2 
                   movlw  0x02
                   call   Button
                   btfsc  CONTROL,1
                   goto   Run_Pause_Start_2 
                   movlw  0x03
                   call   Button
                   btfsc  CONTROL,1
                   goto   Run_Pause_Start_2
                   movlw  0x04
                   call   Button
                   btfsc  CONTROL,1
                   goto   Run_Pause_Start_2
                   movlw  0x05
                   call   Button
                   btfsc  CONTROL,1
                   goto   Run_Pause_Start_2
                   movlw  0x06
                   call   Button
                   btfsc  CONTROL,1
                   goto   Run_Pause_Start_2
                   goto   Run_Pause_Start_1
;**********************************************************************
Run_Pause_Start_2  call   Pause_2
                   call   LCD_WC_Clear
                   bsf    T2CON,TMR2ON
                   call   Pause_1
                   bcf    T2CON,TMR2ON
                   bcf    LED
                   bcf    CONTROL,0
                   bcf    CONTROL,1
                   bcf    CONTROL,2
Run_Pause_Start_3  call   Button
                   btfsc  CONTROL,0
                   goto   Run_F.start
                   call   Pause_2_1
                   btfsc  CONTROL,2
                   goto   Run_Start
                   goto   Run_Pause_Start_3  
;**********************************************************************
;**************************** Run F.start *****************************
;**********************************************************************
Run_F.start        bsf    LED
                   call   LCD_LS_F.start
                   bsf    T2CON,TMR2ON
                   call   Pause_1
                   bcf    T2CON,TMR2ON
                   call   Pause_1
                   bsf    T2CON,TMR2ON
                   call   Pause_1
                   bcf    T2CON,TMR2ON
Run_F.start_1      call   Button
                   btfss  CONTROL,1
                   goto   Run_F.start_1
                   goto   Run_Record
;**********************************************************************
;***************************** Run Start ******************************
;**********************************************************************
Run_Start          bsf    T2CON,TMR2ON
                   bsf    LED
                   clrf   TMR1L
                   clrf   TMR1H
                   clrf   TIME_L
                   clrf   TIME_H
                   bsf    T1CON,TMR1ON 
                   bcf    CONTROL,3
Run_Start_1        call   Cycle_Time
                   btfsc  CONTROL,3
                   bcf    T1CON,TMR1ON                   
                   btfsc  CONTROL,3
                   goto   Run_Record
                   call   Button
                   btfss  CONTROL,0
                   goto   Run_Start_1
                   bcf    T1CON,TMR1ON
                   bcf    T2CON,TMR2ON 
                   incf   COUNT,f
                   call   LCD_Reaction
;**********************************************************************
                   movf   TIME_L,w
                   movwf  COMP_1_L
                   movf   TIME_H,w
                   movwf  COMP_1_H
                   movf   TIME_2_L,w
                   movwf  COMP_2_L
                   movf   TIME_2_H,w
                   movwf  COMP_2_H
                   call   Compare
                   btfss  CONTROL,4
                   goto   Run_Start_2
                   movf   TIME_L,w
                   movwf  TIME_2_L
                   movf   TIME_H,w
                   movwf  TIME_2_H  
;**********************************************************************
                   movf   TIME_2_L,w
                   movwf  COMP_1_L
                   movf   TIME_2_H,w
                   movwf  COMP_1_H
                   movf   TIME_1_L,w
                   movwf  COMP_2_L
                   movf   TIME_1_H,w
                   movwf  COMP_2_H
                   call   Compare
                   btfss  CONTROL,4
                   goto   Run_Start_2
                   movf   TIME_2_L,w
                   movwf  TIME_1_L
                   movf   TIME_2_H,w
                   movwf  TIME_1_H  
                   call   EEPROM_Load_W
;**********************************************************************
Run_Start_2        call   Button
                   btfss  CONTROL,1
                   goto   Run_Start_2
                   bcf    CONTROL,0
                   bcf    CONTROL,1  
Run_Start_3        call   Button
                   btfsc  CONTROL,1
                   goto   Run_Record
                   goto   Run_Start_3
;**********************************************************************
;****************************** Pause 1 ******************************* 
;**********************************************************************
Pause_1           clrf   PAUSE_1
                  clrf   PAUSE_2
Pause_1_1         decfsz PAUSE_1,f
                  goto   Pause_1_1                
                  decfsz PAUSE_2,f
                  goto   Pause_1_1
                  return
;**********************************************************************
;****************************** Pause 2 ******************************* 
;**********************************************************************
Pause_2           movwf  PAUSE_3
                  clrf   PAUSE_1
                  clrf   PAUSE_2
Pause_2_1         decfsz PAUSE_1,f
                  return                
                  decfsz PAUSE_2,f
                  return
                  decfsz PAUSE_3,f
                  return
                  bsf    CONTROL,2
                  return
;**********************************************************************
;******************************* Button ******************************* 
;**********************************************************************
Button             btfsc  BUTTON   
                   bsf    CONTROL,0
                   btfsc  BUTTON
                   return
                   btfsc  CONTROL,0
                   bsf    CONTROL,1
                   return
;**********************************************************************
;***************************** Cycle Time *****************************
;**********************************************************************
Cycle_Time        movlw  0x01
                  subwf  TMR1H,w
                  btfsc  STATUS,Z
                  return
                  movlw  0x01
                  subwf  TMR1H,f 
                  incf   TIME_L,f
                  movlw  0x00
                  subwf  TIME_L,w
                  btfsc  STATUS,Z
                  incf   TIME_H,f
                  movlw  0x27
                  subwf  TIME_H,w
                  btfss  STATUS,Z
                  return
                  movlw  0x10
                  subwf  TIME_L,w
                  btfsc  STATUS,Z
                  bsf    CONTROL,3
                  return
;**********************************************************************
;****************************** Compare *******************************
;**********************************************************************
Compare           bcf    CONTROL,4
                  movf   COMP_1_H,w
                  subwf  COMP_2_H,w
                  btfsc  STATUS,Z
                  goto   Compare_1
                  movf   COMP_1_H,w
                  subwf  COMP_2_H,w
                  btfsc  STATUS,C
                  bsf    CONTROL,4
                  return
Compare_1         movf   COMP_1_L,w
                  subwf  COMP_2_L,w
                  btfsc  STATUS,C
                  bsf    CONTROL,4                 
                  return
;**********************************************************************
;***************************** LCD Record *****************************
;**********************************************************************
LCD_Record        call   LCD_WC_Clear
                  call   LCD_LS_Record
                  call   LCD_WC_2_Line
                  call   LCD_Symbol_
                  call   LCD_Symbol_
                  movf   TIME_1_H,w
                  movwf  VH
                  movf   TIME_1_L,w
                  movwf  VL
                  call   Convert_VHL 
                  call   W_LCD_D1H
                  call   LCD_Symbol_. 
                  call   W_LCD_DL
                  call   LCD_Symbol_
                  call   LCD_Symbol_
                  movf   TIME_2_H,w
                  movwf  VH
                  movf   TIME_2_L,w
                  movwf  VL
                  call   Convert_VHL 
                  call   W_LCD_D1H
                  call   LCD_Symbol_. 
                  call   W_LCD_DL
                  return
;**********************************************************************
;**************************** LCD Reaction ****************************
;**********************************************************************
LCD_Reaction      call   LCD_LS_Reaction
                  call   LCD_WC_2_Line
                  call   LCD_Symbol_
                  call   LCD_Symbol_
                  clrf   VH
                  movf   COUNT,w
                  movwf  VL
                  call   Convert_VHL 
                  call   W_LCD_DL
                  movlw  0x04
                  call   LCD_Step_Clear
                  movf   TIME_H,w
                  movwf  VH
                  movf   TIME_L,w
                  movwf  VL
                  call   Convert_VHL 
                  call   W_LCD_D1H
                  call   LCD_Symbol_. 
                  call   W_LCD_DL
                  return
;**********************************************************************
;************ Convert VH_VL - D4_D3_D2_D1_D0 Write_DD_LCD *************
;**********************************************************************
Convert_VHL clrf  D4
            clrf  D3
            clrf  D2
            clrf  D1
            clrf  D0
;**********************************************************************
Bit0        btfss VL,0
            goto  Bit1
            incf  D0,f
Bit1        btfss VL,1
            goto  Bit2 
            movlw 0x02
            addwf D0,f 
Bit2        btfss VL,2
            goto  Bit3 
            movlw 0x04
            addwf D0,f
Bit3        btfss VL,3
            goto  Bit4 
            movlw 0x08
            addwf D0,f
Bit4        btfss VL,4
            goto  Bit5 
            movlw 0x06
            addwf D0,f
            incf  D1,f
Bit5        btfss VL,5
            goto  Bit6 
            movlw 0x02
            addwf D0,f
            movlw 0x03
            addwf D1,f
Bit6        btfss VL,6
            goto  Bit7 
            movlw 0x04
            addwf D0,f
            movlw 0x06
            addwf D1,f
Bit7        btfss VL,7
            goto  Bit8 
            movlw 0x08
            addwf D0,f
            movlw 0x02
            addwf D1,f
            incf  D2,f
;**********************************************************************
Bit8        btfss VH,0
            goto  Bit9 
            movlw 0x06
            addwf D0,f
            movlw 0x05
            addwf D1,f
            movlw 0x02
            addwf D2,f
Bit9        btfss VH,1
            goto  Bit10 
            movlw 0x02
            addwf D0,f
            incf  D1,f
            movlw 0x05
            addwf D2,f
Bit10       btfss VH,2
            goto  Bit11 
            movlw 0x04
            addwf D0,f
            movlw 0x02
            addwf D1,f
            incf  D3,f
Bit11       btfss VH,3
            goto  Bit12 
            movlw 0x08
            addwf D0,f
            movlw 0x04
            addwf D1,f
            movlw 0x02
            addwf D3,f
Bit12       btfss VH,4
            goto  Bit13 
            movlw 0x06
            addwf D0,f
            movlw 0x09
            addwf D1,f
            movlw 0x04
            addwf D3,f
Bit13       btfss VH,5
            goto  Bit14 
            movlw 0x02
            addwf D0,f
            movlw 0x09
            addwf D1,f
            incf  D2,f
            movlw 0x08
            addwf D3,f
Bit14       btfss VH,6
            goto  Bit15 
            movlw 0x04
            addwf D0,f
            movlw 0x08
            addwf D1,f
            movlw 0x03
            addwf D2,f
            movlw 0x06
            addwf D3,f
            incf  D4,f
Bit15       btfss VH,7
            goto  D0_ready 
            movlw 0x08
            addwf D0,f
            movlw 0x06
            addwf D1,f
            movlw 0x07
            addwf D2,f
            movlw 0x02
            addwf D3,f
            movlw 0x03
            addwf D4,f
;**********************************************************************
D0_ready    movlw 0x0A
            subwf D0,W
            btfss STATUS,C
            goto  D1_ready
            movwf D0
            incf  D1,f
            goto  D0_ready
D1_ready    movlw 0x0A
            subwf D1,W
            btfss STATUS,C
            goto  D2_ready
            movwf D1
            incf  D2,f
            goto  D1_ready          
D2_ready    movlw 0x0A
            subwf D2,W
            btfss STATUS,C
            goto  D3_ready
            movwf D2
            incf  D3,f
            goto  D2_ready  
D3_ready    movlw 0x0A
            subwf D3,W
            btfss STATUS,C
            goto  D4_ready
            movwf D3
            incf  D4,f
            goto  D3_ready 
D4_ready    return 
;**********************************************************************
;********************** Write D-code for LCD **************************
;**********************************************************************
W_LCD_DH    movf  D4,w
            movwf DD
            call  D_Symbol_LCD 
W_LCD_D1H   movf  D3,w
            movwf DD 
            call  D_Symbol_LCD 
            return 
W_LCD_DL    movf  D2,w
            movwf DD 
            call  D_Symbol_LCD 
W_LCD_D2L   movf  D1,w
            movwf DD 
            call  D_Symbol_LCD 
            movf  D0,w
            movwf DD 
            call  D_Symbol_LCD
            return
;**********************************************************************
;************************* D-code for LCD *****************************
;**********************************************************************
D_Symbol_LCD    movlw 0x00
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_0
                movlw 0x01
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_1
                movlw 0x02
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_2
                movlw 0x03
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_3
                movlw 0x04
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_4
                movlw 0x05
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_5
                movlw 0x06
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_6
                movlw 0x07
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_7
                movlw 0x08
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_8
                movlw 0x09
                subwf DD,w
                btfsc STATUS,Z  
                call  LCD_Symbol_9
                return
;**********************************************************************
;******************************* LCD **********************************
;**********************************************************************
LCD_PORTB_Write_L bcf    LCD_DB4
                  bcf    LCD_DB5
                  bcf    LCD_DB6
                  bcf    LCD_DB7
                  btfsc  LCD_BUFFER,0
                  bsf    LCD_DB4  
                  btfsc  LCD_BUFFER,1
                  bsf    LCD_DB5
                  btfsc  LCD_BUFFER,2
                  bsf    LCD_DB6
                  btfsc  LCD_BUFFER,3
                  bsf    LCD_DB7   
                  return   
;**********************************************************************
LCD_PORTB_Write_H bcf    LCD_DB4
                  bcf    LCD_DB5
                  bcf    LCD_DB6
                  bcf    LCD_DB7
                  btfsc  LCD_BUFFER,4
                  bsf    LCD_DB4  
                  btfsc  LCD_BUFFER,5
                  bsf    LCD_DB5
                  btfsc  LCD_BUFFER,6
                  bsf    LCD_DB6
                  btfsc  LCD_BUFFER,7
                  bsf    LCD_DB7   
                  return   
;**********************************************************************
LCD_WC            movwf  LCD_BUFFER
                  bcf    LCD_RS
                  call   LCD_PORTB_Write_H
                  bsf    LCD_E
                  call   Pause_LCD 
                  bcf    LCD_E 
                  call   Pause_LCD
                  return
;**********************************************************************
LCD_WD            movwf  LCD_BUFFER
                  bsf    LCD_RS
                  call   LCD_PORTB_Write_H
                  bsf    LCD_E  
                  nop
                  nop
                  bcf    LCD_E   
                  call   Pause_LCD
                  call   LCD_PORTB_Write_L
                  bsf    LCD_E   
                  nop
                  nop                 
                  bcf    LCD_E 
                  call   Pause_LCD
                  return
;**********************************************************************
LCD_Initializing  movlw  b'00110000'
                  call   LCD_WC
                  call   Pause_LCD
                  movlw  b'00100000'
                  call   LCD_WC
                  movlw  b'10000000'
                  call   LCD_WC            
                  movlw  b'00100000'
                  call   LCD_WC
                  movlw  b'10000000'
                  call   LCD_WC
                  movlw  b'00000000'
                  call   LCD_WC
                  movlw  b'11000000'
                  call   LCD_WC
                  movlw  b'00000000'
                  call   LCD_WC
                  movlw  b'00010000'
                  call   LCD_WC
                  call   Pause_LCD
                  movlw  b'00000000'
                  call   LCD_WC
                  movlw  b'01100000'
                  call   LCD_WC
                  return
;**********************************************************************
LCD_WC_Clear      movlw  b'00000000'
                  call   LCD_WC
                  movlw  b'00010000'
                  call   LCD_WC
                  call   Pause_LCD
                  return
;**********************************************************************
LCD_Step_Clear    movwf  LCD_CYCLE
LCD_Step_Clear_1  call   LCD_Symbol_
                  decfsz LCD_CYCLE,f
                  goto   LCD_Step_Clear_1 
                  return
;**********************************************************************
LCD_WC_2_Line     bcf   LCD_RS
                  movlw b'10110000'
                  call  LCD_WC
                  movlw b'10110000'
                  call  LCD_WC
                  movlw 0x05
                  call  LCD_Step_Clear
                  return
;**********************************************************************
;***************************** Pause LCD ******************************
;**********************************************************************
Pause_LCD        clrf    PAUSE_LCD_1
                 movlw   0x02
                 movwf   PAUSE_LCD_2 
Pause_LCD_1      decfsz  PAUSE_LCD_1,f
                 goto    Pause_LCD_1
                 decfsz  PAUSE_LCD_2,f
                 goto    Pause_LCD_1
                 return   
;**********************************************************************
;**************************** LCD Symbol ******************************
;**********************************************************************
LCD_Symbol_Eng_A  movlw  b'01000001' ;A
                  call   LCD_WD
                  return
LCD_Symbol_Eng_a  movlw  b'01100001' ;a
                  call   LCD_WD
                  return
LCD_Symbol_Eng_B  movlw  b'01000010' ;B
                  call   LCD_WD
                  return
LCD_Symbol_Eng_b  movlw  b'01100010' ;b
                  call   LCD_WD
                  return
LCD_Symbol_Eng_C  movlw  b'01000011' ;C
                  call   LCD_WD 
                  return
LCD_Symbol_Eng_c  movlw  b'01100011' ;c
                  call   LCD_WD 
                  return
LCD_Symbol_Eng_D  movlw  b'01000100' ;D
                  call   LCD_WD
                  return
LCD_Symbol_Eng_d  movlw  b'01100100' ;d
                  call   LCD_WD
                  return
LCD_Symbol_Eng_E  movlw  b'01000101' ;E
                  call   LCD_WD
                  return
LCD_Symbol_Eng_e  movlw  b'01100101' ;e
                  call   LCD_WD
                  return
LCD_Symbol_Eng_F  movlw  b'01000110' ;F
                  call   LCD_WD
                  return
LCD_Symbol_Eng_f  movlw  b'01100110' ;f
                  call   LCD_WD
                  return
LCD_Symbol_Eng_G  movlw  b'01000111' ;G
                  call   LCD_WD
                  return
LCD_Symbol_Eng_g  movlw  b'01100111' ;g
                  call   LCD_WD
                  return
LCD_Symbol_Eng_H  movlw  b'01001000' ;H
                  call   LCD_WD
                  return
LCD_Symbol_Eng_h  movlw  b'01101000' ;h
                  call   LCD_WD
                  return
LCD_Symbol_Eng_I  movlw  b'01001001' ;I
                  call   LCD_WD
                  return
LCD_Symbol_Eng_i  movlw  b'01101001' ;i
                  call   LCD_WD
                  return
LCD_Symbol_Eng_J  movlw  b'01001010' ;J
                  call   LCD_WD
                  return
LCD_Symbol_Eng_j  movlw  b'01101010' ;j
                  call   LCD_WD
                  return
LCD_Symbol_Eng_K  movlw  b'01001011' ;K
                  call   LCD_WD
                  return
LCD_Symbol_Eng_k  movlw  b'01101011' ;k
                  call   LCD_WD
                  return
LCD_Symbol_Eng_L  movlw  b'01001100' ;L
                  call   LCD_WD
                  return
LCD_Symbol_Eng_l  movlw  b'01101100' ;l
                  call   LCD_WD
                  return
LCD_Symbol_Eng_M  movlw  b'01001101' ;M
                  call   LCD_WD
                  return
LCD_Symbol_Eng_m  movlw  b'01101101' ;m
                  call   LCD_WD
                  return
LCD_Symbol_Eng_N  movlw  b'01001110' ;N
                  call   LCD_WD
                  return
LCD_Symbol_Eng_n  movlw  b'01101110' ;n
                  call   LCD_WD
                  return
LCD_Symbol_Eng_O  movlw  b'01001111' ;O
                  call   LCD_WD
                  return
LCD_Symbol_Eng_o  movlw  b'01101111' ;o
                  call   LCD_WD
                  return
LCD_Symbol_Eng_P  movlw  b'01010000' ;P
                  call   LCD_WD
                  return
LCD_Symbol_Eng_p  movlw  b'01110000' ;p
                  call   LCD_WD
                  return
LCD_Symbol_Eng_Q  movlw  b'01010001' ;Q
                  call   LCD_WD
                  return
LCD_Symbol_Eng_q  movlw  b'01110001' ;q
                  call   LCD_WD
                  return
LCD_Symbol_Eng_R  movlw  b'01010010' ;R
                  call   LCD_WD
                  return
LCD_Symbol_Eng_r  movlw  b'01110010' ;r
                  call   LCD_WD
                  return
LCD_Symbol_Eng_S  movlw  b'01010011' ;S
                  call   LCD_WD
                  return
LCD_Symbol_Eng_s  movlw  b'01110011' ;s
                  call   LCD_WD
                  return
LCD_Symbol_Eng_T  movlw  b'01010100' ;T
                  call   LCD_WD
                  return
LCD_Symbol_Eng_t  movlw  b'01110100' ;t
                  call   LCD_WD
                  return
LCD_Symbol_Eng_U  movlw  b'01010101' ;U
                  call   LCD_WD
                  return
LCD_Symbol_Eng_u  movlw  b'01110101' ;u
                  call   LCD_WD
                  return
LCD_Symbol_Eng_V  movlw  b'01010110' ;V
                  call   LCD_WD
                  return
LCD_Symbol_Eng_v  movlw  b'01110110' ;v
                  call   LCD_WD
                  return
LCD_Symbol_Eng_W  movlw  b'01010111' ;W
                  call   LCD_WD
                  return
LCD_Symbol_Eng_w  movlw  b'01110111' ;w
                  call   LCD_WD
                  return
LCD_Symbol_Eng_X  movlw  b'01011000' ;X
                  call   LCD_WD
                  return
LCD_Symbol_Eng_x  movlw  b'01111000' ;x
                  call   LCD_WD
                  return
LCD_Symbol_Eng_Y  movlw  b'01011001' ;Y
                  call   LCD_WD
                  return
LCD_Symbol_Eng_y  movlw  b'01111001' ;y
                  call   LCD_WD
                  return
LCD_Symbol_Eng_Z  movlw  b'01011010' ;Z
                  call   LCD_WD
                  return
LCD_Symbol_Eng_z  movlw  b'01111010' ;z
                  call   LCD_WD
                  return
;**********************************************************************
LCD_Symbol_Rus_A  movlw  b'01000001' ;A
                  call   LCD_WD
                  return
LCD_Symbol_Rus_a  movlw  b'01100001' ;a
                  call   LCD_WD
                  return
LCD_Symbol_Rus_B  movlw  b'10100000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_b  movlw  b'10110010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_V  movlw  b'01000010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_v  movlw  b'10110011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_G  movlw  b'10100001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_g  movlw  b'10110100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_D  movlw  b'11100000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_d  movlw  b'11100011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_E  movlw  b'01000101' ;E
                  call   LCD_WD
                  return
LCD_Symbol_Rus_e  movlw  b'01100101' ;e
                  call   LCD_WD
                  return
LCD_Symbol_Rus_YO movlw  b'10100010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_yo movlw  b'10110101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_J  movlw  b'10100011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_j  movlw  b'10110110' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_Z  movlw  b'10100100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_z  movlw  b'10110111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_I  movlw  b'10100101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_i  movlw  b'10111000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_IK movlw  b'10100110' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_ik movlw  b'10111001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_K  movlw  b'01001011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_k  movlw  b'10111010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_L  movlw  b'10100111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_l  movlw  b'10111011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_M  movlw  b'01001101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_m  movlw  b'10111100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_N  movlw  b'01001000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_n  movlw  b'10111101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_O  movlw  b'01001111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_o  movlw  b'01101111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_P  movlw  b'10101000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_p  movlw  b'10111110' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_R  movlw  b'01010000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_r  movlw  b'01110000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_S  movlw  b'01000011' ;
                  call   LCD_WD 
                  return
LCD_Symbol_Rus_s  movlw  b'01100011' ;
                  call   LCD_WD 
                  return
LCD_Symbol_Rus_T  movlw  b'01010100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_t  movlw  b'10111111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_U  movlw  b'10101001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_u  movlw  b'01111001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_F  movlw  b'10101010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_f  movlw  b'11100100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_H  movlw  b'01011000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_h  movlw  b'01111000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_C  movlw  b'11100001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_c  movlw  b'11100101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_CH movlw  b'10101011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_ch movlw  b'11000000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_SH movlw  b'10101100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_sh movlw  b'11000001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_SHA movlw  b'11100010' ;
                   call   LCD_WD
                   return
LCD_Symbol_Rus_sha movlw  b'11100110' ;
                   call   LCD_WD
                   return
LCD_Symbol_Rus_TZ movlw  b'10101101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_tz movlw  b'11000010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_Y  movlw  b'10101110' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_y  movlw  b'11000011' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_MZ movlw  b'01100010' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_mz movlw  b'11000100' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_EE movlw  b'10101111' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_ee movlw  b'11000101' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_YU movlw  b'10110000' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_yu movlw  b'11000110' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_YA movlw  b'10110001' ;
                  call   LCD_WD
                  return
LCD_Symbol_Rus_ya movlw  b'11000111' ;
                  call   LCD_WD
                  return
;**********************************************************************
LCD_Symbol_0      movlw  b'00110000' ;0
                  call   LCD_WD 
                  return
LCD_Symbol_1      movlw  b'00110001' ;1
                  call   LCD_WD 
                  return
LCD_Symbol_2      movlw  b'00110010' ;2
                  call   LCD_WD 
                  return
LCD_Symbol_3      movlw  b'00110011' ;3
                  call   LCD_WD 
                  return
LCD_Symbol_4      movlw  b'00110100' ;4
                  call   LCD_WD 
                  return
LCD_Symbol_5      movlw  b'00110101' ;5
                  call   LCD_WD 
                  return
LCD_Symbol_6      movlw  b'00110110' ;6
                  call   LCD_WD 
                  return
LCD_Symbol_7      movlw  b'00110111' ;7
                  call   LCD_WD 
                  return
LCD_Symbol_8      movlw  b'00111000' ;8
                  call   LCD_WD 
                  return
LCD_Symbol_9      movlw  b'00111001' ;9
                  call   LCD_WD 
                  return
;**********************************************************************
LCD_Symbol_       movlw  b'00010000' ;
                  call   LCD_WD
                  return
LCD_Symbol_.      movlw  b'00101110' ;.
                  call   LCD_WD 
                  return
LCD_Symbol_..     movlw  b'00111010' ;:
                  call   LCD_WD 
                  return
LCD_Symbol_scb    movlw  b'00101000' ;(
                  call   LCD_WD 
                  return
LCD_Symbol_sce    movlw  b'00101001' ;)
                  call   LCD_WD 
                  return
LCD_Symbol_spb    movlw  b'01011011' ;[
                  call   LCD_WD 
                  return
LCD_Symbol_spe    movlw  b'01011101' ;]
                  call   LCD_WD
                  return
;**********************************************************************
;************************ LCD Link Symbols ****************************
;**********************************************************************
LCD_LS_Record     call   LCD_Symbol_
                  call   LCD_Symbol_Eng_R
                  call   LCD_Symbol_Eng_e
                  call   LCD_Symbol_Eng_c
                  call   LCD_Symbol_Eng_o
                  call   LCD_Symbol_Eng_r
                  call   LCD_Symbol_Eng_d
                  call   LCD_Symbol_
                  call   LCD_Symbol_
                  call   LCD_Symbol_Rus_R
                  call   LCD_Symbol_Rus_e
                  call   LCD_Symbol_Rus_k
                  call   LCD_Symbol_Rus_o
                  call   LCD_Symbol_Rus_r
                  call   LCD_Symbol_Rus_d                  
                  return     
;**********************************************************************
LCD_LS_F.start    call   LCD_Symbol_Eng_F    
                  call   LCD_Symbol_.
                  call   LCD_Symbol_Eng_s
                  call   LCD_Symbol_Eng_t
                  call   LCD_Symbol_Eng_a
                  call   LCD_Symbol_Eng_r
                  call   LCD_Symbol_Eng_t 
                  call   LCD_Symbol_
                  call   LCD_Symbol_
                  call   LCD_Symbol_Rus_F 
                  call   LCD_Symbol_.
                  call   LCD_Symbol_Rus_s
                  call   LCD_Symbol_Rus_t
                  call   LCD_Symbol_Rus_a
                  call   LCD_Symbol_Rus_r
                  call   LCD_Symbol_Rus_t
                  return
;**********************************************************************
LCD_LS_Reaction   call   LCD_Symbol_Eng_R    
                  call   LCD_Symbol_Eng_e
                  call   LCD_Symbol_Eng_a
                  call   LCD_Symbol_Eng_c
                  call   LCD_Symbol_Eng_t
                  call   LCD_Symbol_Eng_i
                  call   LCD_Symbol_Eng_o
                  call   LCD_Symbol_Eng_n
                  call   LCD_Symbol_
                  call   LCD_Symbol_Rus_R
                  call   LCD_Symbol_Rus_e
                  call   LCD_Symbol_Rus_a
                  call   LCD_Symbol_Rus_k
                  call   LCD_Symbol_Rus_c
                  call   LCD_Symbol_Rus_i
                  call   LCD_Symbol_Rus_ya
                  return
;********************************************************************** 
;******************************* EEPROM *******************************
;********************************************************************** 
; 0x00 - TIME_1_L
; 0x01 - TIME_1_H       
;********************************************************************** 
EEPROM_W         movf    RDATA,W
                 bsf     STATUS,RP0 
                 bsf     EECON1,WREN 
                 movwf   EEDATA 
                 bcf     STATUS,RP0
                 movf    RADR,W
                 bsf     STATUS,RP0 
                 movwf   EEADR
                 bcf     INTCON,GIE
                 movlw   0x55
                 movwf   EECON2
                 movlw   0xAA
                 movwf   EECON2
                 bsf     EECON1,WR
                 bsf     INTCON,GIE
                 bcf     STATUS,RP0
                 call    EEPROM_R           
                 subwf   RDATA,W
                 btfss   STATUS,Z   
                 goto    EEPROM_W
                 return
;**********************************************************************
EEPROM_R         movf    RADR,W
                 bsf     STATUS,RP0
                 bcf     STATUS,RP1
                 movwf   EEADR
                 bsf     EECON1,RD
                 movf    EEDATA,W 
                 bcf     STATUS,RP0
                 return 
;********************************************************************** 
;*************************** EEPROM Load R ****************************
;********************************************************************** 
EEPROM_Load_R    movlw   0x00
                 movwf   RADR
                 call    EEPROM_R
                 movwf   TIME_1_L 
                 movlw   0x01
                 movwf   RADR
                 call    EEPROM_R
                 movwf   TIME_1_H  
                 return
;********************************************************************** 
;*************************** EEPROM Load W ****************************
;********************************************************************** 
EEPROM_Load_W    movlw   0x00
                 movwf   RADR
                 movf    TIME_1_L,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 movlw   0x01
                 movwf   RADR
                 movf    TIME_1_H,w
                 movwf   RDATA 
                 call    EEPROM_W
                 call    Pause_1
                 return
;**********************************************************************

; initialize eeprom locations

	ORG	0x2100
	DE	0x0F, 0x27


	END                       ; directive 'end of program'

